/**
  *  \file client/tiles/taskedittile.cc
  *  \brief WAutoTaskEditorTile tile
  *
  *  (c) 2001-2018 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/tiles/taskedittile.h"
#include "client/screens/taskscreen.h"
#include "client/widgets/taskeditlist.h"
#include "game/objsel.h"
#include "ui/layout.h"
#include "ui/util.h"
#include "ui/widgets/framegroup.h"

namespace {
    enum {
        NLINES = 18,
        WIDTH  = 315
    };
}

/** Constructor.
    \param selection Object selection. Should be a WAutoTaskEditorTile descendant. */
WAutoTaskEditorTile::WAutoTaskEditorTile(GObjectSelection& selection)
    : WTile(GfxPoint(WIDTH, NLINES*font_heights[FONT_NORMAL]+2)),
      atsel(dynamic_cast<WAutoTaskObjectSelection*>(&selection)),
      atlist(0)
{
    init();
}

WAutoTaskEditorTile::~WAutoTaskEditorTile()
{ }

void
WAutoTaskEditorTile::init()
{
    // Make sure we can receive focus
    setState(st_Disabled, false);

    // Default layout
    setLayout(LeftJustified, 0, true);

    // Create list only when we're actually observing a WAutoTaskObjectSelection
    if (atsel != 0) {
        // Wrap it into a UIFrameGroup to draw the frame; we cannot draw in the
        // drawData routine because we don't get to the coordinates.
        atlist = &holder.add(new WAutoTaskEditorList(atsel->getEditor(), NLINES, WIDTH));
        addControl(GfxRect(0, 0, WIDTH, NLINES*font_heights[FONT_NORMAL]+2),
                   UIFrameGroup::wrap(holder, *atlist));
        atlist->sig_moved.add(this, &WAutoTaskEditorTile::onListMoved);
        atlist->setFocusedWidget();

        // Accept changes on the list
        conn_task_changed = atsel->sig_task_changed.add(this, &WAutoTaskEditorTile::onTaskChanged);
    }
}

void
WAutoTaskEditorTile::drawData(GfxCanvas& /*can*/)
{ }

void
WAutoTaskEditorTile::onListMoved()
{
    /* List has been moved. Update cursor in WAutoTaskObjectSelection. */
    atsel->setCursor(atlist->getCurrent());
}

void
WAutoTaskEditorTile::onTaskChanged(bool newTask)
{
    /* Task has been changed. Update the task and the cursor. */
    atlist->update(newTask);
    atlist->scrollTo(atsel->getCursor(), 1);
}
