/**
  *  \file client/widgets/engineinfo.cc
  *  \brief WEngineInfoDiagram widget
  *
  *  (c) 2001-2018 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/widgets/engineinfo.h"
#include "gfx/gfxcontext.h"
#include "util/nls.h"
#include "gfx/palette.h"
#include "ui/uiglobal.h"
#include "ui/util.h"
#include "game/spec.h"

/*************************** WEngineInfoDiagram **************************/

WEngineInfoDiagram::WEngineInfoDiagram()
    : UIWidget(0), engine(1)
{ }

void
WEngineInfoDiagram::drawContent(GfxCanvas& can)
{
    int bottom = getExtent().getBottomY();
    int center = getExtent().getCenter().x;

    GfxContext ctx;
    ctx.useCanvas(can);
    ctx.useColorScheme(getSkin());
    ctx.useFont(*fonts[FONT_SMALL]);
    ctx.setTextAlign(1, 0);
    ctx.setColor(UIColor::tc_Static);

    outTextFormatted(ctx, center, bottom - 2*font_heights[FONT_SMALL], extent.w,
                     _("Fuel used per\nlightyear"));

    GfxRect diagram_pos(getExtent());
    diagram_pos.h -= 2*font_heights[FONT_SMALL] + 1;

    drawFrameDown(can, diagram_pos);
    diagram_pos.grow(-1, -1);
    drawSolidBar(can, diagram_pos, standard_colors[COLOR_BLACK]);
    diagram_pos.grow(-1, -1);

    ctx.useColorScheme(GfxStandardColorScheme::instance);
    ctx.setColor(COLOR_WHITE);
    ctx.setTextAlign(0, 0);
    outText(ctx, diagram_pos.x, diagram_pos.getBottomY() - font_heights[FONT_SMALL],
            _("Warp \xE2\x86\x92"));
    diagram_pos.h -= font_heights[FONT_SMALL]+1;

    /* Diagram */
    int prev_ff = 0;
    int prev_x = 0;
    for (int warp = 1; warp <= 10; ++warp) {
        int new_x = warp*diagram_pos.w / 10;
        int new_ff = warp == 10 ? 1000 : getEngine(engine).getFuelFactor(warp) / (warp*warp);
        if (new_ff > 999)
            new_ff = 999;

        int delta_x = new_x - prev_x;

        /* draw the bars */
        for (int dx = 1; dx < delta_x; ++dx) {
            int this_ff    = (dx*new_ff + (delta_x-dx)*prev_ff) / delta_x;
            int this_color = COLOR_STATUS+15 - (16*this_ff)/1000;
            int this_y     = this_ff*diagram_pos.h/1000;
            ctx.setColor(this_color);
            drawVLine(ctx,
                      diagram_pos.x + dx + prev_x,
                      diagram_pos.getBottomY() - this_y,
                      diagram_pos.getBottomY());
        }
        prev_x = new_x;
        prev_ff = new_ff;
    }

    can.update(getExtent());
}

void
WEngineInfoDiagram::getLayoutInfo(LayoutInfo& info)
{
    /* The diagram is 2 (top) + 100 (size) + 1 (gap) + text + 2 (bottom)
       + 1 (gap) + 2*text */
    info.pref_size = info.min_size =
        GfxPoint(102, 106 + 3*font_heights[FONT_SMALL]);
    info.flags = 0;
}

void
WEngineInfoDiagram::setEngine(int n)
{
    if (n != engine) {
        engine = n;
        drawWidget(false);
    }
}
