/**
  *  \file game/team.h
  *  \brief Players and Teams
  *
  *  This module contains player properties and accessors, which only
  *  affect display, not rules. This includes
  *  - current player
  *  - team assignments
  *  - data transfer settings
  *
  *  (c) 2001-2018 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_TEAM_H
#define PCC_V2_GAME_TEAM_H

#include "cpluslib/string.h"

class Stream;
class Signal0;
class GAlliances;

/** Player-to-Player Relation. */
enum GPlayerRelation {
    is_Me,                      ///< It's we.
    is_Ally,                    ///< It's an ally.
    is_Enemy                    ///< It's an enemy.
};

int             getPlayerId();
int             getRealPlayerId();
void            setPlayerIds(int eff, int real);
GPlayerRelation getPlayerRelation(int n);

int             getPlayerTeam(int player);
void            setPlayerTeam(int player, int team);
void            removePlayerTeam(int player);
int             getNumTeamMembers(int team);
string_t        getTeamName(int team);
void            setTeamName(int team, string_t name);
bool            isNamedTeam(int team);
bool            isTeamConfigured();

void            syncTeamsFromAlliances(const GAlliances& allies);

extern Signal0  sig_team_change;
extern Signal0  sig_player_change;

extern const struct ModuleInitializer mod_team[];

#endif
