/**
  *  \file int/if/objcontext.h
  *  \brief Interpreter Game Interface: Object Context
  *
  *  (c) 2001-2018 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_INT_IF_OBJCONTEXT_H
#define PCC_V2_INT_IF_OBJCONTEXT_H

#include "int/context.h"

class GObjectType;
class GUniverse;

/** Object context (GObjectType-based).
    This implements the generic parts of a context for an object
    type that is backed by a GObjectType.

    Initialisation:
    - the "Get(x)" function maps to "new IntObjectContext(type, x)"
    - the "Make" function maps to "new IntObjectContext(type, 0)->initFirst". */
class IntObjectContext : public IntContext {
 public:
    // Construction and Destruction:
    IntObjectContext(GObjectType& type, int index);
    ~IntObjectContext();
    IntContext* initFirst();

    // Implementation of IntContext methods:
    bool     next();
    GObject* getObject();

    // Own Methods:
    int getCurrentIndex() const;

    GUniverse* getCurrentUniverse() const;

 private:
    GObjectType& type;
    int index;
};

/** Constructor.
    \param type Underlying object type.
    \param index Index. Need not be a valid index for the type. */
inline
IntObjectContext::IntObjectContext(GObjectType& type, int index)
    : type(type),
      index(index)
{ }

/** Get current index. */
inline int
IntObjectContext::getCurrentIndex() const
{
    return index;
}

#endif
